#!/usr/bin/env python
# -*- coding: utf-8 -*-

# #####################################################################
# Copyright (c) 2005 Jiří Popek <jiri.popek@gmail.com>
#
# name        : pDict
# description : Python and Qt dictionary search
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# LICENCE:
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# author : Jiří Popek
# email  : jiri.popek@gmail.com
# date   : 24.1.2005
#
# last modify: 4.8.05
# history:
#       19.7.05 simplified - now using startswith() function, added
#               items to queue object, works as simple Thread instead
#               of persistent object which creates new thread everytime
#               is called
#       27.4.05 removed qt dependent objects from this file
# #####################################################################

from threading import Thread
from globals import *

__author__ = u'Jiří Popek'
__doc__ = """searching core of the application"""

########################################################################
## core of searching
class DictionarySearch(Thread):
    """ main searching class """
    def __init__(self, txt, queue, endFunc, datafile, index):
        """ initial function
        Params:
            txt - list [0]-text, [1]-text length (need for unicode strings because for example len(u"ř")=2
            queue - Queue object to add new items to
            endFunc - fuction called by thread to indicate finish of searching
            datafile - name of file containing dictionary data
            index - dict object (contains dictionary indexed data)
        """
        
        self.queue      = queue
        self.stopFlag   = False
        self.endFunc    = endFunc
        self.text       = txt
        self.datafile   = datafile
        self.index      = index
        self.finished   = False
        
        if not DEBUG:
            Thread.__init__(self, target=self.worker)
            self.setName("pDictSearchingThread")
        
        # start the thread
        if DEBUG:
            self.worker()
        else:
            self.setDaemon(True)
            self.start()
    
    def shutdown(self):
        """ stops searching thread """
        self.stopFlag = True
    
    def worker(self):
        """searching core"""
        self.stopFlag = 0
        skipall = False
        text = self.text[0] # string
        ln = self.text[1] # length
        self.found = False # if found first occurence
        
        # fetch the index from dictionary
        idx = text[:DEPTH]
        self.offset = self.index.get(idx, -1)
        
        if self.offset==-1:
            skipall=True  # haven't found first character
        
        items_count = 0
        if skipall==False:
            f = open(self.datafile, 'r')
            f.seek(self.offset)
            
            text = text.lower()
            lt = len(text)
            ln = f.readline()
            while(ln!='' and not self.stopFlag):
                ln = ln.rstrip() # remove end of line
                ln = unicode(ln, 'utf-8')
                lst = ln.split(TAB)
                key = lst[0].lower()
                
                if key.startswith(text):
                    # call handler to add new item
                    self.queue.put(lst, False)
                    
                    items_count += 1          # increment items counter
                    if self.found==False:
                        self.found = True     # found 1st occurence
                else:
                    if self.found==True:
                        # found occurence before, but new one doesn't start with text
                        break
                    else:
                        if ln<=DEPTH: 
                            if not key.startswith(text[:-1]):
                                break
                        else:
                            if not key.startswith(text[:DEPTH]):
                                break
                
                ln = f.readline()
            f.close()
        
        if not self.stopFlag:
            self.endFunc(items_count)
